/*
-------------------------------------------------------------------------------
    This file is a part of the Vst Preset Generator
    Copyright (C) 2007  Francois Mazen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-------------------------------------------------------------------------------
  Name: parammanager.cpp
  Author: Francois Mazen
  Date: 28/05/07 16:25
  Description: ParamManager class contain data about parameters (type of
               random generation, values...) and interface with the table view
-------------------------------------------------------------------------------
*/

#include <QtGui>

#include "parammanager.h"


//-----------------------------------------------------------------------------
ParamManager::ParamManager(QObject *parent) : QAbstractTableModel(parent) //QStandardItemModel(parent)
{
	//create the random module, and init it
	vpgRandom = new VpgRandom();
	QTime midnight(0, 0, 0);
    vpgRandom->initSeed(midnight.secsTo(QTime::currentTime()));


}

ParamManager::~ParamManager()
{
    delete vpgRandom;
}

//-----------------------------------------------------------------------------
int ParamManager::rowCount(const QModelIndex &parent) const
{
 	return paramList.size();

}

//-----------------------------------------------------------------------------
int ParamManager::columnCount(const QModelIndex &parent) const
{
 	return 4; //name|type|val1|val2
}

//-----------------------------------------------------------------------------
QVariant ParamManager::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
         return QVariant();

    paramStruct buffer = paramList.at(index.row());
    QString stringBuffer;


    if(role == Qt::DisplayRole)
    {
        if(index.column() == 0)
        {
             return(buffer.name);
	    }
	    else if(index.column() == 1)
	    {
             if(buffer.randomMode == uniform)
                  return (tr("uniform"));
             if(buffer.randomMode == normal)
                  return (tr("normal"));
             if(buffer.randomMode == constant)
                  return (tr("constant"));
        }
        else if(index.column() == 2)
        {

             if(buffer.randomMode == uniform)
             {
 		          stringBuffer.setNum(buffer.min);
 		          stringBuffer.prepend( tr("min: "));
                  return (stringBuffer);
             }
             if(buffer.randomMode == normal)
             {
 		          stringBuffer.setNum(buffer.mid);
 		          stringBuffer.prepend ( tr("mid: "));
                  return (stringBuffer);
             }
             if(buffer.randomMode == constant)
             {
 		          stringBuffer.setNum(buffer.val);
 		          stringBuffer.prepend ( tr("val: "));
                  return (stringBuffer);
             }
        }
        else if(index.column() == 3)
        {
             if(buffer.randomMode == uniform)
             {
 		          stringBuffer.setNum(buffer.max);
 		          stringBuffer.prepend ( tr("max: "));
                  return (stringBuffer);
             }
             if(buffer.randomMode == normal)
             {
 		          stringBuffer.setNum(buffer.dev);
 		          stringBuffer.prepend ( tr("dev: "));
                  return (stringBuffer);
             }
             if(buffer.randomMode == constant)
             {
 		          return " ";
             }
        }
        else
            return QVariant();
    }
    else if (role == Qt::DecorationRole)
    {
	 	 if(index.column()==1)
         {
	         if(buffer.randomMode == uniform)
                  return (QIcon(":/images/uniform.png"));
             if(buffer.randomMode == normal)
                  return (QIcon(":/images/normal.png"));
             if(buffer.randomMode == constant)
                  return (QIcon(":/images/constant.png"));


            return QVariant();
         }
    }
    else if (role == Qt::EditRole)
    {
        if(index.column() == 1)
        {
            return buffer.randomMode;
        }
    }

    return QVariant();
}

//-----------------------------------------------------------------------------
QVariant ParamManager::headerData(int section, Qt::Orientation orientation,
                                  int role) const
{
   if (role != Qt::DisplayRole)
         return QVariant();

     if (orientation == Qt::Horizontal)
     {
         switch(section)
         {
               case 0 : return QString(tr("name"));
               case 1 : return QString(tr("random mode"));
               case 2 : return QString(tr("val 1"));
               case 3 : return QString(tr("val 2"));
               default : return QString();
         }
     }
     else
         return QString("%1").arg(section + 1);
}

//-----------------------------------------------------------------------------
Qt::ItemFlags ParamManager::flags(const QModelIndex &index) const
{
     if (!index.isValid())
         return Qt::ItemIsEnabled;

     //the first column can not be edited
     //neither the last column in constant mode
     if((index.column() == 3) &&
		(paramList.at(index.row()).randomMode == constant))
		return QAbstractItemModel::flags(index);

     if(index.column() > 0)
         return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;

     return QAbstractItemModel::flags(index);
}

//-----------------------------------------------------------------------------
bool ParamManager::setData(const QModelIndex &index,
                            const QVariant &value, int role)
{
     paramStruct buffer;
     if (index.isValid() && role == Qt::EditRole) {

         buffer = paramList.at(index.row());
         if((index.column() == 1) &&
            ((eRandomMode)value.toInt() < randomModeMax))
         {
              buffer.randomMode = (eRandomMode)value.toInt();
		 }
		 if(index.column() == 2)
		 {
              switch(buffer.randomMode)
              {
                   case uniform :
				   		if(value.toDouble() > buffer.max)
				   		{ //avoid min > max
			                buffer.min = buffer.max;
			            }
			            else
			            {
				   		buffer.min = value.toDouble();
						}
						break;
                   case normal :	buffer.mid = value.toDouble(); break;
                   case constant :  buffer.val = value.toDouble(); break;
                   default : break;
			  }
		 }
		 if(index.column() == 3)
		 {
              switch(buffer.randomMode)
              {
                   case uniform :
				   		if(value.toDouble() < buffer.min)
				   		{//avoid max < min
                            buffer.max = buffer.min;
			            }
			            else
			            {
                        buffer.max = value.toDouble();
						}
						break;
                   case normal :	buffer.dev = value.toDouble(); break;
                   default : break;
			  }
         }
         paramList.replace(index.row(),buffer);
         emit dataChanged(index, index);

         return true;
     }
     return false;
}

//-----------------------------------------------------------------------------
void ParamManager::clear()
{
 	 paramList.clear();
}

//-----------------------------------------------------------------------------
void ParamManager::add(int id, paramStruct param)
{
 	 paramList.replace(id - 1, param);
 	 emit dataChanged(index(id-1, 0), index(id-1, 3));
}

//-----------------------------------------------------------------------------
void ParamManager::setParamNb(int nb)
{
 	 paramList.resize(nb);
 	 emit layoutChanged();
}

//-----------------------------------------------------------------------------
float ParamManager::getRandomValue(int id_param)
{
 	  paramStruct buffer = paramList.at(id_param);

      switch(buffer.randomMode)
      {
      case constant:
           return (buffer.val);
           break;
      case normal:
           //mu and deviation
           return (vpgRandom->getNormalValue(buffer.mid, buffer.dev));
           break;
      case uniform:
           //min-max
      default:
           return (vpgRandom->getUniformValue(buffer.min, buffer.max));
           break;

      }

      return 0.f;
}

//-----------------------------------------------------------------------------
void ParamManager::ChangeRandomMethodForAll(eRandomMode method)
{
    paramStruct dummyParam;
    int i;

    dummyParam.min = 0;
    dummyParam.max = 1;
    dummyParam.mid = 0.5;
    dummyParam.dev = 0.1;
    dummyParam.val = 0.5;


    switch(method)
    {
        case uniform :
            dummyParam.randomMode = uniform;

            break;
        case normal :
            dummyParam.randomMode = normal;
            break;
        case constant :
            dummyParam.randomMode = constant;
            break;
        case random :
        default :       break;
    }

    for(i=0; i< paramList.size(); i++)
    {
        if(method == random)
        {
            dummyParam.randomMode = (eRandomMode)(int)vpgRandom->getUniformValue(0, randomModeMax);
            dummyParam.min = vpgRandom->getUniformValue(0,1);
            dummyParam.max = vpgRandom->getUniformValue(dummyParam.min,1);
            dummyParam.mid = vpgRandom->getUniformValue(0,1);
            dummyParam.dev = vpgRandom->getUniformValue(0,0.2);
            dummyParam.val = vpgRandom->getUniformValue(0,1);
        }

        //we save the name of the parameter
        dummyParam.name = paramList[i].name;
        paramList.replace(i,dummyParam);

    }

    emit dataChanged(index(0, 0), index(paramList.size() - 1, 3));



}

